/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2008, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.test.deployers.support.deployer;

import javax.naming.Context;
import javax.naming.NamingException;

import org.jboss.weld.integration.deployer.jndi.AbstractJndiBinderDeployer;

/**
 * Mock checkable jndi binder deployer.
 *
 * @author <a href="mailto:ales.justin@jboss.org">Ales Justin</a>
 */
public class CheckableJndiBinderDeployer extends AbstractJndiBinderDeployer<Object>
{
   public static Context ROOT = new MockContext();

   public CheckableJndiBinderDeployer()
   {
      super(Object.class);
   }

   @Override
   protected Context createContext() throws NamingException
   {
      return ROOT;
   }

   public void bind(Object bootstrapBean, String deploymentUnitName) throws NamingException
   {
      Context context = getBeanManagerContext().createSubcontext(deploymentUnitName);
      context.bind("bootstrap", bootstrapBean);
   }

   public void unbind(Object bootstrapBean, String deploymentUnitName) throws NamingException
   {
      Context context = (Context)getBeanManagerContext().lookup(deploymentUnitName);
      context.unbind("bootstrap");

      getBeanManagerContext().destroySubcontext(deploymentUnitName);
   }
}